#include "StdAfx.h"
#include ".\subcondition.h"
#include "Globals.h"

IMPLEMENT_SERIAL( SubCondition, CObject, 1 )


SubCondition::SubCondition(int col, int comp, double refValue)
: size(1)
, orType(false)
, referenceType(0)
, textRef(false)
{
	column = col;
	comparator = comp;
	referenceValue = refValue;
}

SubCondition::SubCondition(SubCondition *copySubCondition)
{
	column = copySubCondition->column;
	comparator = copySubCondition->comparator;
	orType = copySubCondition->orType;
	referenceValue = copySubCondition->referenceValue;
	referenceType = copySubCondition->referenceType;
	referenceValueText = copySubCondition->referenceValueText;
	size = copySubCondition->size;
	textRef = copySubCondition->textRef;
}

SubCondition::SubCondition(int col, int comp, CString refValueText)
: size(1)
, orType(false)
, referenceType(0)
, textRef(true)

{
	column = col;
	comparator = comp;
	referenceValueText = refValueText;
}

SubCondition::~SubCondition(void)
{
}

void SubCondition::changeValues(int col, int comp, double refValue, int refType)
{
	textRef = false;
	column = col;
	comparator = comp;
	referenceType = refType;
	referenceValue = refValue;
}

void SubCondition::changeValues(int col, int comp, CString refValueText, int refType)
{
	textRef = true;
	column = col;
	referenceType = refType;
	comparator = comp;
	referenceValueText = refValueText;
}

BOOL SubCondition::check(WTextFile *theTextFile, int lineNumber)
{
	BOOL result = false;

	// Check whether line is not empty ...
	if( ( wcscmp( theTextFile->at( lineNumber, column )->c_str( ), L"" ) == 0 ) )
		return false;

	double critValue = _wtof( theTextFile->at( lineNumber, column )->c_str( ) );
	CString critValueText; critValueText  = theTextFile->at( lineNumber, column )->c_str( );

	double comparisonValue;
	CString comparisonValueText;

	// Choose correct value for comparison:
	// constant: the value of the variable referenceValue or referenceValueText;
	// column #: interpret the variable referenceValue as column number
	if( referenceType == 0 ) // constant
	{
		comparisonValue = referenceValue;
		comparisonValueText = referenceValueText;
	}
	else if( referenceType == 1 ) // column #
	{
		comparisonValue = _wtof( theTextFile->at( lineNumber, referenceValue-1 )->c_str( ) );
		comparisonValueText = theTextFile->at( lineNumber, referenceValue-1 )->c_str( );
	}
	
	//////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////// No textRef//////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////////
	if (! textRef)
	{
		switch( comparator )
		{
			// If the difference between critValue and comparisonValue is exactly equal to COMPARE_RES,
			// this is counted as equal.
			case -2:
				//if ( critValue < comparisonValue)
				if( critValue - comparisonValue < -COMPARE_RES )
					result = true;
				break;
			case -1:
				//if ( critValue <= comparisonValue)
				if
				( 
					(critValue - comparisonValue < -COMPARE_RES) || // <
					(fabs( critValue - comparisonValue ) <= COMPARE_RES) // ==
				)
					result = true;
				break;
			case 0:
				//if ( critValue == comparisonValue)
				if( fabs( critValue - comparisonValue ) <= COMPARE_RES )
					result = true;
				break;
			case 1:
				//if ( critValue >= comparisonValue)
				if
				( 
					(critValue - comparisonValue > COMPARE_RES) || // >
					(fabs( critValue - comparisonValue ) <= COMPARE_RES) // ==
				)
					result = true;
				break;
			case 2:
				//if ( critValue > comparisonValue)
				if( (critValue - comparisonValue > COMPARE_RES) )
					result = true;
				break;
			case 10:
				//if ( critValue != comparisonValue)
				if( fabs( critValue - comparisonValue ) > COMPARE_RES)
					result = true;
				break;
			case 20:
				/*if ((( critValue > 0.0 ) && ( comparisonValue > 0.0 ))
					|| (( critValue < 0.0 ) && ( comparisonValue < 0.0 )))*/
				if ((( critValue > COMPARE_RES ) && ( comparisonValue > COMPARE_RES ))
					|| (( critValue < -COMPARE_RES ) && ( comparisonValue < -COMPARE_RES )))
					result = true;
				break;
			case 21:
				/*if ((( critValue > 0.0 ) && ( comparisonValue < 0.0 ))
					|| (( critValue < 0.0 ) && ( comparisonValue > 0.0 )))*/
				if ((( critValue > COMPARE_RES ) && ( comparisonValue < -COMPARE_RES ))
					|| (( critValue < -COMPARE_RES ) && ( comparisonValue > COMPARE_RES )))
					result = true;
				break;
		}
	}
	//////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////// textRef//////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////////
	else // textRef
	{
		switch( comparator )
		{
			case -2:
				if ( critValueText < comparisonValueText )
					result = true;
				break;
			case -1:
				if ( critValueText <= comparisonValueText )
					result = true;
				break;
			case 0:
				if ( critValueText == comparisonValueText )
					result = true;
				break;
			case 1:
				if ( critValueText >= comparisonValueText )
					result = true;
				break;
			case 2:
				if ( critValueText > comparisonValueText )
					result = true;
				break;
			case 10:
				if ( critValueText != comparisonValueText )
					result = true;
				break;
		}
	}
	
	return result;
}

void SubCondition::setTextRef(BOOL text)
{
	textRef = text;
}

void SubCondition::Serialize( CArchive& archive )
{
    CObject::Serialize( archive );

	if( archive.IsStoring( ) )
	{
		archive << column << comparator << referenceType << referenceValue << referenceValueText
		<< textRef << size << orType;
	}
	else
	{
		archive >> column >> comparator >> referenceType >> referenceValue >> referenceValueText
		>> textRef >> size >> orType;
	}
}

InputError SubCondition::checkValidInputs( WTextFile * theTextFile )
{
	InputError inputError;
	
	if( (column < 0) || (column > (theTextFile->returnNumberOfColumns() - 1)) )
		inputError = handleInputError( INPUT_ERROR_COLUMN );

	if( referenceType == 1 ) // column #
	{
		if( ((referenceValue-1) < 0) || ((referenceValue-1) > (theTextFile->returnNumberOfColumns() - 1)) )
			inputError = handleInputError( INPUT_ERROR_COLUMN_REFERENCE );
	}

	if( textRef )
	{
		switch( comparator )
		{
			case 20: // equal sign
				inputError = handleInputError( INPUT_ERROR_TEXTREF_EQUAL_SIGN );
				break;
			case 21: // unequal sign
				inputError = handleInputError( INPUT_ERROR_TEXTREF_UNEQUAL_SIGN );
				break;
		}
	}
	
	return inputError;
}
